import { defineStore } from 'pinia'
import { type ToastServiceMethods } from 'primevue'

import type { Inventory, AppConfig } from 'src/types/data'
import { saveConfig } from './utils/io'

interface InventoryState {
  filePath: string,
  data: Inventory | null,
}

const fileTree = (state: InventoryState) => state.filePath.split(/(\\)|(\/)/gm)

export const useAppStore = defineStore('appStore', {
  state: () => ({
    currentInventory: {
      filePath: '',
      data: null as Inventory | null,
    } as InventoryState,
    currentConfig: {} as AppConfig
  }),
  getters: {
    fileTree(state) {
      return fileTree(state.currentInventory)
    },
    fileDirectory(state) {
      const split = fileTree(state.currentInventory)
      return state.currentInventory.filePath.replace(split[split.length - 1], '')
    },
    fileName(state) {
      const split = fileTree(state.currentInventory)
      return split[split.length - 1]
    },
  },
  actions: {
    async setInventory(path: string, data: Inventory, toast: ToastServiceMethods) {
      this.currentInventory.filePath = path
      this.currentInventory.data = data

      if (!this.currentConfig.recentFiles) {
        this.currentConfig.recentFiles = []
        this.currentConfig.recentFiles.push(path)
      } else if (!this.currentConfig.recentFiles.includes(path)) {
        this.currentConfig.recentFiles.push(path)
      }

      await saveConfig(this.currentConfig, toast)
    },
    async removeRecentFile(path: string, toast: ToastServiceMethods) {
      const index = this.currentConfig.recentFiles!.findIndex(other => other === path)
      if (index > -1) {
        this.currentConfig.recentFiles = this.currentConfig.recentFiles?.filter((_, i) => i !== index)
        await saveConfig(this.currentConfig, toast)
      }
    }
  }
})

export type AppStoreDefinition = Omit<
  ReturnType<typeof useAppStore>,
  keyof ReturnType<typeof defineStore>
>
